package idea.jartool;

import com.intellij.codeHighlighting.BackgroundEditorHighlighter;
import com.intellij.ide.structureView.StructureViewBuilder;
import com.intellij.openapi.fileEditor.FileEditor;
import com.intellij.openapi.fileEditor.FileEditorLocation;
import com.intellij.openapi.fileEditor.FileEditorState;
import com.intellij.openapi.fileEditor.FileEditorStateLevel;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.util.JDOMUtil;
import com.intellij.openapi.util.Key;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.ui.FilteringListModel;
import idea.jartool.actions.JarAction;
import org.jdom.Document;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */
public class JarEditor extends JPanel implements ActionListener,
                                                 PropertyChangeListener,
                                                 FileEditor
{
    VirtualFile virtualFile;
    ClassSelectionPanel classSelectionPanel;

    FilteringListModel fm;

    JButton saveButton = new JButton();
    JPanel buttonPanel = new JPanel();
    JButton makeButton = new JButton();
    JButton cleanButton = new JButton();
    JButton refreshButton = new JButton();
    private JarProject jarProject;
    private Project project;

    JScrollPane messageAreaScroll = new JScrollPane();
    JTextArea messageArea = new JTextArea();
    JSplitPane mainSplitPane = new JSplitPane();


    //Construct the frame
    public JarEditor(VirtualFile virtualFile)
    {
        this.virtualFile = virtualFile;
        try
        {
            jbInit();
            setupGUI();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    //Component initialization
    private void jbInit() throws Exception
    {
        classSelectionPanel = new ClassSelectionPanel();
        this.setDebugGraphicsOptions(0);
        this.setLayout(new GridBagLayout());
        saveButton.setMaximumSize(new Dimension(80, 21));
        saveButton.setMinimumSize(new Dimension(80, 21));
        saveButton.setPreferredSize(new Dimension(80, 21));
        saveButton.setRequestFocusEnabled(true);
        saveButton.setMnemonic('S');
        saveButton.setText("Save");
        makeButton.setText("Make");
        makeButton.setMnemonic('S');
        makeButton.setRequestFocusEnabled(true);
        makeButton.setPreferredSize(new Dimension(80, 21));
        makeButton.setMinimumSize(new Dimension(80, 21));
        makeButton.setMaximumSize(new Dimension(80, 21));
        cleanButton.setText("Clean");
        cleanButton.setMnemonic('S');
        cleanButton.setRequestFocusEnabled(true);
        cleanButton.setPreferredSize(new Dimension(80, 21));
        cleanButton.setMinimumSize(new Dimension(80, 21));
        cleanButton.setMaximumSize(new Dimension(80, 21));
        refreshButton.setText("Refresh");
        refreshButton.setMnemonic('S');
        refreshButton.setRequestFocusEnabled(true);
        refreshButton.setPreferredSize(new Dimension(80, 21));
        refreshButton.setMinimumSize(new Dimension(80, 21));
        refreshButton.setMaximumSize(new Dimension(80, 21));

        messageArea.setSize(300, 200);
        messageArea.setBackground(Color.WHITE);
        messageArea.setEditable(false);
        messageArea.setAutoscrolls(true);
        messageArea.setBorder(BorderFactory.createEtchedBorder());
        messageAreaScroll.getViewport().add(messageArea);
        mainSplitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
        mainSplitPane.setTopComponent(classSelectionPanel);
        mainSplitPane.setBottomComponent(messageAreaScroll);
        mainSplitPane.setResizeWeight(.6);

        mainSplitPane.setOneTouchExpandable(true);

        this.add(mainSplitPane, new GridBagConstraints(0, 0, 1, 1, 1, 1, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(6, 6, 6, 6), 0, 0));
        this.add(buttonPanel, new GridBagConstraints(0, 2, 1, 1, 1, 0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(6, 6, 6, 6), 0, 0));


        buttonPanel.add(cleanButton, null);
        buttonPanel.add(makeButton, null);
        buttonPanel.add(saveButton, null);
        buttonPanel.add(refreshButton, null);
    }

    void setupGUI()
    {
        refreshButton.addActionListener(this);
        cleanButton.addActionListener(this);
        saveButton.addActionListener(this);
        makeButton.addActionListener(this);
        showMessageBox(false);
    }

    public void setData(Project project, JarProject jarProject)
    {
        this.project = project;
        this.jarProject = jarProject;
    }

    public void refresh()
    {
        classSelectionPanel.setData(project, jarProject);
    }

    public VirtualFile getVirtualFile()
    {
        return virtualFile;
    }

    public void save()
    {
        classSelectionPanel.save();

        try
        {
            Document doc = jarProject.createDoc();
            JDOMUtil.writeDocument(doc, virtualFile.getPath(), "\n");
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    public void actionPerformed(ActionEvent e)
    {
        if (e.getSource() == saveButton)
        {
            save();
        }
        else if (e.getSource() == refreshButton)
        {
            refresh();
        }
        else if (e.getSource() == cleanButton)
        {
            clean();
        }
        else if (e.getSource() == makeButton)
        {
            save();
            new JarAction().createJar(jarProject, project);
        }
    }

    private void clean()
    {
        File archive = new File(jarProject.getArchiveDir() + File.separator + jarProject.getArchiveName());

        if (archive.exists())
        {
            archive.delete();
        }

        archive = new File(jarProject.getSourceDir() + File.separator + jarProject.getSourceName());

        if (archive.exists())
        {
            archive.delete();
        }

    }

    public void propertyChange(PropertyChangeEvent e)
    {
    }


    public JarProject getJarProject()
    {
        return jarProject;
    }

    public JComponent getComponent()
    {
        return this;
    }

    public JComponent getPreferredFocusedComponent()
    {
        return this;
    }

    public String getName()
    {
        return "Jar Editor";
    }

    public FileEditorState getState(FileEditorStateLevel fileEditorStateLevel)
    {
        return new FileEditorState()
        {
            public boolean canBeMergedWith(FileEditorState fileEditorState, FileEditorStateLevel fileEditorStateLevel)
            {
                return true;
            }
        };

    }

    public void showMessageBox(boolean show)
    {
        messageAreaScroll.setVisible(show);
        if (show)
        {
            mainSplitPane.setBottomComponent(messageAreaScroll);
        }
    }

    public void clearMessages()
    {
        messageArea.setText("");
    }

    public void appendMessage(String message)
    {
        messageArea.append(message);
    }

    public void setState(FileEditorState fileEditorState)
    {
    }

    public boolean isModified()
    {
        return true;
    }

    public boolean isValid()
    {
        return true;
    }

    public void selectNotify()
    {
    }

    public void deselectNotify()
    {
    }

    public BackgroundEditorHighlighter getBackgroundHighlighter()
    {
        return null;
    }

    public FileEditorLocation getCurrentLocation()
    {
        return null;
    }

    public StructureViewBuilder getStructureViewBuilder()
    {
        return null;
    }

    public <T> T getUserData(Key<T> key)
    {
        return null;
    }

    public <T> void putUserData(Key<T> key, T t)
    {

    }

    public void dispose()
    {
    }
}
