package idea.jartool;

import com.intellij.openapi.components.ApplicationComponent;
import com.intellij.openapi.fileEditor.*;
import com.intellij.openapi.module.Module;
import com.intellij.openapi.module.ModuleManager;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.roots.ModuleRootManager;
import com.intellij.openapi.roots.OrderRootType;
import com.intellij.openapi.util.JDOMUtil;
import com.intellij.openapi.vfs.VirtualFile;
import org.jdom.Element;

import java.util.Hashtable;
import java.util.Vector;
import java.io.File;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */

public class JarPlugin implements ApplicationComponent, FileEditorProvider
{
    public static final String COMPONENT_NAME = "JarPlugin";
    public static final String ARCHIVE_DIR = "archive_projects";

    private static final String extensions = "jarprj";

    private Hashtable<String, JarProject> jarProjects = new Hashtable<String, JarProject>();
    private Hashtable<JarProject, JarEditor> jarProjectEditors = new Hashtable<JarProject, JarEditor>();

    static
    {
    }

    public JarPlugin()
    {
    }


    public String getComponentName()
    {
        return COMPONENT_NAME;
    }

    public void initComponent()
    {
        //nothing to init
    }

    public void disposeComponent()
    {
    }

    //FileEditorProvider
    public boolean accept(Project project, VirtualFile virtualFile)
    {
        return extensions.equalsIgnoreCase(virtualFile.getExtension());
    }

    /**
     * loads all the classes of the project including all the modules. jar librariy files
     * also being returned by the open API hence they are being ignored at this point. Only the classes
     * are taken in
     * @param project
     * @param jarProject
     */
    public Vector<SelectionItem> loadClasses(Project project, JarProject jarProject)
    {
        ModuleManager moduleManager = ModuleManager.getInstance(project);

        //find module roots and load classe
        Module projectModule = moduleManager.findModuleByName(project.getName());
        ModuleRootManager mrm = ModuleRootManager.getInstance(projectModule);
        VirtualFile vfiles[] = mrm.getFiles(OrderRootType.COMPILATION_CLASSES);

        //variable is declared here and sent due to the recursive behaviour
        Vector<SelectionItem> classes = new Vector<SelectionItem>();
        for (VirtualFile vfile : vfiles)
        {
            if (!vfile.getPresentableUrl().trim().endsWith(".jar"))
            {
                //send to load the classes recursively starting from this directory
                loadClasses(classes, jarProject, vfile);
            }
        }

        return classes;
    }

    /**
     * a recursively called method, classes variable is filled by calling the method recursively
     * this method is called in loading the build project and on refresh
     * @param classes, data vectory holding the classes
     * @param jarProject, jar proejcte
     * @param dir
     */
    public void loadClasses(Vector<SelectionItem> classes, JarProject jarProject, VirtualFile dir)
    {
//        FileManagerImpl fm = new FileManagerImpl();
//        fm.
        //if the directory is selected, then all the classes of the directory should be loading in automatically
        boolean selectAllInDir = jarProject.getDirectories().contains(dir.getPresentableUrl());

        VirtualFile[] files = dir.getChildren();

        for (VirtualFile file : files)
        {
            if (file.isDirectory())
            {
                //for each directory in the parent call the load method recursively
                loadClasses(classes, jarProject, file);
            }
            else
            {
                if (selectAllInDir && !jarProject.getClasses().contains(file.getPresentableUrl()))
                {
                    /**
                     * add to the jar project if the class is selected or if the directory is selected
                     *
                     * this is important to have, as after the creation of the jarprj file, if there are
                     * any new clases added, by this method at the loadin gof the project they will be
                     * selected in to the build project. and user will have the facility of just to selecting the
                     * directories and not to worry on the classes
                     */
                    jarProject.getClasses().addElement(file.getPresentableUrl());
                }

                /**
                 * classe entries are kept by the file namem, this way all the $ named inner clasees can also
                 * be selected one by one,
                 *
                 * selection is set beased ont he directory selection or the selection on the classe individually
                 */
                classes.addElement(new SelectionItem((selectAllInDir ||
                                                      jarProject.getClasses().contains(file.getPresentableUrl())),
                                                     file.getPresentableUrl()));
            }
        }

    }

    /**
     * Loading the data fromthe .jarprj file in to the JarProject object. the jarprj file is being already
     * selected and only them will be sent to this place
     * @param project
     * @param virtualFile
     */
    public JarProject loadJarProject(Project project, VirtualFile virtualFile)
    {
        JarProject jarProject = new JarProject();
        jarProject.setProjectPathDirs(project.getBaseDir().getPath());

        try
        {
            org.jdom.Document doc = JDOMUtil.loadDocument( new File( virtualFile.getPath() ) );
            jarProject.read(doc);
            jarProjects.put(virtualFile.getPath(), jarProject);

            loadClasses(project, jarProject);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return jarProject;
    }

    public FileEditor createEditor(Project project, VirtualFile virtualFile)
    {
        JarEditor jarEditor = new JarEditor(virtualFile);
        try
        {
            jarEditor.setData(project, loadJarProject(project, virtualFile));
            jarEditor.refresh();
            jarProjectEditors.put(jarEditor.getJarProject(), jarEditor);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return jarEditor;
    }

    public void disposeEditor(FileEditor fileEditor)
    {
        if (fileEditor instanceof JarEditor)
        {
            ((JarEditor) fileEditor).save();
            JarProject jarProject = jarProjects.remove(((JarEditor) fileEditor).getVirtualFile().getPath());
            jarProjectEditors.remove(jarProject);
        }
    }

    public FileEditorState readState(Element element, Project project, VirtualFile virtualFile)
    {
        return new FileEditorState()
        {
            public boolean canBeMergedWith(FileEditorState fileEditorState, FileEditorStateLevel fileEditorStateLevel)
            {
                return false;
            }
        };
    }

    public void writeState(FileEditorState fileEditorState, Project project, Element element)
    {
    }

    public String getEditorTypeId()
    {
        return "jarprj";
    }

    public FileEditorPolicy getPolicy()
    {
        return FileEditorPolicy.HIDE_DEFAULT_EDITOR;
    }

    /**
     * when multiple project are being opened they are kept in a list which they could be loaded by the
     * jarprj file name
     * @param key
     * @return JarProject
     */
    public JarProject getJarProject(String key)
    {
        if (jarProjects.containsKey(key))
        {
            return jarProjects.get(key);
        }

        return null;
    }

    /**
     * sabe the values onthe UI to the object
     * @param jarPrj
     */
    public void save(JarProject jarPrj)
    {
        if (jarProjectEditors.containsKey(jarPrj))
        {
            (jarProjectEditors.get(jarPrj)).save();
        }
    }

    /**
     * shows or hides the message window at the bottom of the UI
     * @param jarPrj
     * @param show
     */
    public void showMessageBox(JarProject jarPrj, boolean show)
    {
        if (jarProjectEditors.containsKey(jarPrj))
        {
            (jarProjectEditors.get(jarPrj)).showMessageBox(show);
        }
    }

    /**
     * clears the output message window shown at the bottom of the UI
     * @param jarPrj
     */
    public void clearMessages(JarProject jarPrj)
    {
        if (jarProjectEditors.containsKey(jarPrj))
        {
            (jarProjectEditors.get(jarPrj)).clearMessages();
        }
    }

    /**
     * appending the string messages to the output message window
     * @param jarPrj
     * @param message
     */
    public void appendMessage(JarProject jarPrj, String message)
    {
        if (jarProjectEditors.containsKey(jarPrj))
        {
            (jarProjectEditors.get(jarPrj)).appendMessage(message);
        }
    }

}
