package idea.jartool;

import com.intellij.openapi.util.InvalidDataException;
import com.intellij.openapi.util.WriteExternalException;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Attribute;
import org.apache.tools.ant.taskdefs.Manifest;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.io.File;

/**
 * Created by IntelliJ IDEA.
 * User: ruwan
 * Date: 19-Sep-2004
 * Time: 10:53:30
 */
public class JarProject
{
    public static String COMPONENT_NAME = "JAR";

    public static String VERSION = "VERSION";
    public static String ARCHIVE_NAME = "ARCHIVE_NAME";
    public static String ARCHIVE_DIRECTORY = "ARCHIVE_DIRECTORY";
    public static String MAIN_CLASS = "MAIN_CLASS";
    public static String INCLUDE_ALL_PROJECT_RESOURCE = "INCLUDE_ALL_PROJECT_RESOURCE";
    public static String BUILD_BEFORE_ARCHIVING = "BUILD_BEFORE_ARCHIVING";
    public static String BUILD_SOURCE = "BUILD_SOURCE";
    public static String SINGLE_SOURCE = "SINGLE_SOURCE";
    public static String SOURCE_NAME = "SOURCE_NAME";
    public static String SOURCE_DIR = "SOURCE_DIR";
    public static String SIGNED_LIBRARY_PATH = "SIGNED_LIBRARY_PATH";

    public static String ATTR_SIGNED = "SIGNED";

    public static String ELEMENT = "ELEMENT";
    public static String CLASSES = "CLASSES";
    public static String SOURCES = "SOURCES";
    public static String RESOURCES = "RESOURCES";
    public static String LIBRARIES = "LIBRARIES";
    public static String DIRECTORIES = "DIRECTORIES";
    public static String MANIFEST = "MANIFEST";

    private String version = "";
    private String archiveName;
    private String mainClass;
    private String archiveDir;
    private String signedLibraryPath;
    private boolean buildBeforeArchiving;
    private boolean buildSourceArchive;
    private boolean singleSourceArchive;
    private boolean includeAllProjectResources;
    private String sourceName;
    private String sourceDir;

    private Vector<String> classes;
    private Vector<String> sources;
    private Vector<String> resources;
    private Vector<Library> libraries;
    private Vector<String> directories;
    private Vector<String> projectPathDirs;
    private Vector<Manifest.Attribute> manifestEntries;

    public JarProject()
    {
        classes = new Vector<String>();
        sources = new Vector<String>();
        resources = new Vector<String>();
        libraries = new Vector<Library>();
        directories = new Vector<String>();
        manifestEntries = new Vector<Manifest.Attribute>(30);
        mainClass = "";
        archiveDir = "";
        archiveName = "";
        archiveDir = "";
        signedLibraryPath = "";
    }


    public Vector<String> getProjectPathDirs()
    {
        return projectPathDirs;
    }

    public void setProjectPathDirs(Vector<String> projectPathDirs)
    {
        this.projectPathDirs = projectPathDirs;
    }

    public void setProjectPathDirs(String projectPath)
    {
        File f = new File(projectPath);

        projectPathDirs = new Vector<String>(1);

        while (f.getParentFile() != null)
        {
            projectPathDirs.addElement(f.getAbsolutePath());
            f = f.getParentFile();
        }
    }

    public String getArchiveName()
    {
        return archiveName;
    }

    public void setArchiveName(String archiveName)
    {
        this.archiveName = archiveName;
    }

    public String getMainClass()
    {
        return mainClass;
    }

    public void setMainClass(String mainClass)
    {
        this.mainClass = mainClass;
    }

    public String getArchiveDir()
    {
        return archiveDir;
    }

    public void setArchiveDir(String archiveDir)
    {
        this.archiveDir = archiveDir;
    }

    public Vector<String> getClasses()
    {
        return classes;
    }

    public void setClasses(Vector<String> classes)
    {
        this.classes = classes;
    }

    public Vector<String> getResources()
    {
        return resources;
    }

    public void setResources(Vector<String> resources)
    {
        this.resources = resources;
    }

    public Vector<Library> getLibraries()
    {
        return libraries;
    }

    public void setLibraries(Vector<Library> libraries)
    {
        this.libraries = libraries;
    }

    public boolean isBuildSourceArchive()
    {
        return buildSourceArchive;
    }

    public void setBuildSourceArchive(boolean buildSourceArchive)
    {
        this.buildSourceArchive = buildSourceArchive;
    }

    public Vector<String> getDirectories()
    {
        return directories;
    }

    public void setDirectories(Vector<String> directories)
    {
        this.directories = directories;
    }

    public String getSourceName()
    {
        return sourceName;
    }

    public void setSourceName(String sourceName)
    {
        this.sourceName = sourceName;
    }

    public String getSourceDir()
    {
        return sourceDir;
    }

    public void setSourceDir(String sourceDir)
    {
        this.sourceDir = sourceDir;
    }

    public Vector<String> getSources()
    {
        return sources;
    }

    public void setSources(Vector<String> sources)
    {
        this.sources = sources;
    }

    public boolean isSingleSourceArchive()
    {
        return singleSourceArchive;
    }

    public void setSingleSourceArchive(boolean singleSourceArchive)
    {
        this.singleSourceArchive = singleSourceArchive;
    }

    public boolean isIncludeAllProjectResources()
    {
        return includeAllProjectResources;
    }

    public void setIncludeAllProjectResources(boolean includeAllProjectResources)
    {
        this.includeAllProjectResources = includeAllProjectResources;
    }

    public boolean isBuildBeforeArchiving()
    {
        return buildBeforeArchiving;
    }

    public void setBuildBeforeArchiving(boolean buildBeforeArchiving)
    {
        this.buildBeforeArchiving = buildBeforeArchiving;
    }

    public String getSignedLibraryPath()
    {
        return signedLibraryPath;
    }

    public void setSignedLibraryPath(String signedLibraryPath)
    {
        this.signedLibraryPath = signedLibraryPath;
    }

    public String getVersion()
    {
        return version;
    }

    public void setVersion(String version)
    {
        this.version = version;
    }


    public Vector<Manifest.Attribute> getManifestEntries()
    {
        return manifestEntries;
    }

    public void setManifestEntries(Vector<Manifest.Attribute> manifestEntries)
    {
        this.manifestEntries = manifestEntries;
    }

    public String getSignedLibraryManifestValue()
    {
        String val = "";

        for (int i = libraries.size() - 1; i >= 0; i--)
        {
            Library library = libraries.elementAt(i);

            if (library.isSigned())
            {
                if (signedLibraryPath != null && !signedLibraryPath.equals(""))
                {
                    val += signedLibraryPath + File.separator;
                }
                val += library.getFileName().trim() + " ";//(i == 0 ? "" : ";");//do not add ; to the last item
            }
        }
        return val;
    }

    public void read(Document doc) throws InvalidDataException
    {
        try
        {
            Element root = doc.getRootElement();
            if (root.getChild(JarProject.VERSION) != null)
            {
                version = root.getChild(JarProject.VERSION).getTextTrim();
            }

            archiveName = root.getChild(JarProject.ARCHIVE_NAME).getTextTrim();
            archiveDir = PathConverter.toAbsolutePath(projectPathDirs, root.getChild(JarProject.ARCHIVE_DIRECTORY).getTextTrim());
            mainClass = root.getChild(JarProject.MAIN_CLASS).getTextTrim();
            buildSourceArchive = root.getChild(JarProject.BUILD_SOURCE).getTextTrim().equalsIgnoreCase("YES");
            singleSourceArchive = root.getChild(JarProject.SINGLE_SOURCE).getTextTrim().equalsIgnoreCase("YES");
            sourceDir = PathConverter.toAbsolutePath(projectPathDirs, root.getChild(JarProject.SOURCE_DIR).getTextTrim());
            sourceName = root.getChild(JarProject.SOURCE_NAME).getTextTrim();
            includeAllProjectResources = root.getChild(JarProject.INCLUDE_ALL_PROJECT_RESOURCE).getTextTrim().equalsIgnoreCase("YES");
            buildBeforeArchiving = root.getChild(JarProject.BUILD_BEFORE_ARCHIVING).getTextTrim().equalsIgnoreCase("YES");

            if (root.getChild(JarProject.SIGNED_LIBRARY_PATH) != null)
            {
                signedLibraryPath = root.getChild(JarProject.SIGNED_LIBRARY_PATH).getTextTrim();
            }
            //classes
            Element elm = root.getChild(JarProject.CLASSES);
            List list = elm.getChildren(JarProject.ELEMENT);
            Iterator iterator = list.listIterator();
            while (iterator.hasNext())
            {
                classes.addElement(PathConverter.toAbsolutePath(projectPathDirs, ((Element) iterator.next()).getTextTrim()));
            }

            //jars
            elm = root.getChild(JarProject.LIBRARIES);
            list = elm.getChildren(JarProject.ELEMENT);
            iterator = list.iterator();
            while (iterator.hasNext())
            {
                Element tempElm = (Element) iterator.next();
                libraries.add(new Library(PathConverter.toAbsolutePath(projectPathDirs, tempElm.getTextTrim()),
                    (tempElm.getAttribute(ATTR_SIGNED) != null &&
                        tempElm.getAttribute(ATTR_SIGNED).getValue().equals("true"))));
            }

            elm = root.getChild(JarProject.SOURCES);
            list = elm.getChildren(JarProject.ELEMENT);
            iterator = list.iterator();
            while (iterator.hasNext())
            {
                sources.add(PathConverter.toAbsolutePath(projectPathDirs, ((Element) iterator.next()).getTextTrim()));
            }

            //resourrces
            elm = root.getChild(JarProject.RESOURCES);
            list = elm.getChildren(JarProject.ELEMENT);
            iterator = list.iterator();
            while (iterator.hasNext())
            {
                resources.add(PathConverter.toAbsolutePath(projectPathDirs, ((Element) iterator.next()).getTextTrim()));
            }

            elm = root.getChild(JarProject.DIRECTORIES);
            list = elm.getChildren(JarProject.ELEMENT);
            iterator = list.iterator();
            while (iterator.hasNext())
            {
                directories.add(PathConverter.toAbsolutePath(projectPathDirs, ((Element) iterator.next()).getTextTrim()));
            }

            /**
             * following 2 attributes are taken from the jar project and not fromthe table,
             * but these are displayed in the manifest table
             */
            manifestEntries.addElement(new Manifest.Attribute(Manifest.ATTRIBUTE_MANIFEST_VERSION, version));
            manifestEntries.addElement(new Manifest.Attribute("Main-Class", mainClass));

            elm = root.getChild(JarProject.MANIFEST);
            if (elm != null)
            {
                list = elm.getChildren(JarProject.ELEMENT);
                iterator = list.iterator();
                while (iterator.hasNext())
                {
                    Element item = (Element) iterator.next();
                    Attribute attr = item.getAttribute("KEY");
                    if (attr != null)
                    {
                        if (!(attr.getValue().equalsIgnoreCase("Main-Class") ||
                            attr.getValue().equalsIgnoreCase(Manifest.ATTRIBUTE_MANIFEST_VERSION)))
                        {
                            manifestEntries.add(new Manifest.Attribute(attr.getValue(),
                                item.getAttribute("VALUE") != null ? item.getAttribute("VALUE").getValue() : ""));
                        }
                    }
                }
            }
            /**
             * to keep the table rows created dummi entries added, amximum 50 entries can be put in to the maifest file
             */
            int count = 100 - manifestEntries.size();

            if (count > 0)
            {
                for (int i = 0; i < count; i++)
                {
                    manifestEntries.addElement(new Manifest.Attribute("", ""));
                }
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    public Document createDoc() throws WriteExternalException
    {
        Document doc = new Document();
        Element root = new Element(JarProject.COMPONENT_NAME);
        doc.setRootElement(root);

        Element e = new Element(JarProject.VERSION);
        e.addContent(version);
        root.addContent(e);

        e = new Element(JarProject.ARCHIVE_NAME);
        e.addContent(archiveName);
        root.addContent(e);

        e = new Element(JarProject.ARCHIVE_DIRECTORY);
        e.addContent(PathConverter.toRelativePath(projectPathDirs, archiveDir));
        root.addContent(e);

        e = new Element(JarProject.MAIN_CLASS);
        e.addContent(mainClass);
        root.addContent(e);

        e = new Element(JarProject.BUILD_SOURCE);
        e.addContent(buildBeforeArchiving ? "YES" : "NO");
        root.addContent(e);

        e = new Element(JarProject.SINGLE_SOURCE);
        e.addContent(singleSourceArchive ? "YES" : "NO");
        root.addContent(e);

        e = new Element(JarProject.BUILD_BEFORE_ARCHIVING);
        e.addContent(buildBeforeArchiving ? "YES" : "NO");
        root.addContent(e);

        e = new Element(JarProject.SOURCE_DIR);
        e.addContent(PathConverter.toRelativePath(projectPathDirs, sourceDir));
        root.addContent(e);

        e = new Element(JarProject.SOURCE_NAME);
        e.addContent(sourceName);
        root.addContent(e);

        e = new Element(JarProject.INCLUDE_ALL_PROJECT_RESOURCE);
        e.addContent(includeAllProjectResources ? "YES" : "NO");
        root.addContent(e);

        e = new Element(JarProject.SIGNED_LIBRARY_PATH);
        e.addContent(signedLibraryPath);
        root.addContent(e);

        /**
         * Classes
         */
        e = new Element(JarProject.CLASSES);
        root.addContent(e);
        for (int i = 0; i < classes.size(); i++)
        {
            Element tmp = new Element(JarProject.ELEMENT);
            tmp = tmp.addContent(PathConverter.toRelativePath(projectPathDirs, classes.elementAt(i)));
            e.addContent(tmp);
        }

        /**
         * libraries
         */
        e = new Element(JarProject.LIBRARIES);
        root.addContent(e);
        for (Library library : libraries)
        {
            Element tmp = new Element(JarProject.ELEMENT);
            tmp.setAttribute(ATTR_SIGNED, library.isSigned() ? "true" : "false");
            tmp = tmp.addContent(PathConverter.toRelativePath(projectPathDirs, library.getPath()));
            e.addContent(tmp);
        }

        /**
         *
         */
        e = new Element(JarProject.SOURCES);
        root.addContent(e);
        for (int i = 0; i < sources.size(); i++)
        {
            Element tmp = new Element(JarProject.ELEMENT);
            tmp = tmp.addContent(PathConverter.toRelativePath(projectPathDirs, sources.elementAt(i)));
            e.addContent(tmp);
        }

        /**
         * Resources
         */
        e = new Element(JarProject.RESOURCES);
        root.addContent(e);
        for (int i = 0; i < resources.size(); i++)
        {
            Element tmp = new Element(JarProject.ELEMENT);
            tmp = tmp.addContent(PathConverter.toRelativePath(projectPathDirs, resources.elementAt(i)));
            e.addContent(tmp);
        }

        /**
         * Selected Dirs
         */
        e = new Element(JarProject.DIRECTORIES);
        root.addContent(e);
        for (int i = 0; i < directories.size(); i++)
        {
            Element tmp = new Element(JarProject.ELEMENT);
            tmp = tmp.addContent(PathConverter.toRelativePath(projectPathDirs, directories.elementAt(i)));
            e.addContent(tmp);
        }

        e = new Element(JarProject.MANIFEST);
        root.addContent(e);
        for (int i = 0; i < manifestEntries.size(); i++)
        {
            Manifest.Attribute attr = manifestEntries.elementAt(i);

            if (!attr.getName().trim().equals(""))
            {
                Element tmp = new Element(JarProject.ELEMENT);
                tmp.setAttribute("KEY", attr.getName());

                if (attr.getName().equalsIgnoreCase("Main-Class"))
                {
                    tmp.setAttribute("VALUE", mainClass);
                }
                else if (attr.getName().equalsIgnoreCase(Manifest.ATTRIBUTE_MANIFEST_VERSION))
                {
                    tmp.setAttribute("VALUE", version);
                }
                else
                {
                    tmp.setAttribute("VALUE", attr.getValue());
                }
                e.addContent(tmp);
            }
        }

        return doc;
    }
}
