package idea.jartool;

import java.util.Properties;
import java.util.jar.JarOutputStream;
import java.util.jar.Manifest;
import java.util.jar.Attributes;
import java.util.jar.JarEntry;
import java.io.*;

public class Test
{

    private static Properties jarManifestProperties;

    /**
     * Method to convert the given files to a JAR file.
     *
     * @param jarName the <code>String</code> name of the new JAR file.
     * @param jarDir  the <code>File</code> representing the destination
     *                directory for the new JAR file.
     * @param files   all the files to be included in the JAR file.
     */
    public static boolean jar( String jarName,
                               File jarDir,
                               File[] files )
    {
        return jar( jarName, jarDir, null, files );
    }

    /**
     * Method to convert the given files to a JAR file.
     *
     * @param jarName      the <code>String</code> name of the new JAR file.
     * @param jarDir       the <code>File</code> representing the destination
     *                     directory for the new JAR file.
     * @param manifestFile the <code>File</code> to be used as a Manifest entry
     *                     for the new JAR file.
     * @param files        all the files to be included in the JAR file.
     */
    public static boolean jar( String jarName,
                               File jarDir,
                               File manifestFile,
                               File[] files )
    {

        Manifest manifest = null;
        JarOutputStream jarOut = null;

        try
        {
            //1. Create the Manifest
            if ( manifestFile == null || !manifestFile.exists() )
            {
                //manifest = new Manifest();
                System.out.println( "Manifest file is null or does not exist!" );
                System.out.println( "Creating new manifest file..." );
                manifest = new Manifest();
                //String mainClassName = files[0].getName().substring(0,
                //        files[0].getName().indexOf("."));
                //manifest.getMainAttributes().putValue(
                //                        "Main-Class", mainClassName);
            }
            else
            {

                InputStream manifestIn = new FileInputStream( manifestFile );
                //manifest = new Manifest(manifestIn);

                jarManifestProperties = new Properties();
                jarManifestProperties.load( manifestIn );

                String mainClassName =
                    jarManifestProperties.getProperty( "Main-Class" );
                System.out.println( "Manifest Main-Class: " + mainClassName );

                manifest = new Manifest();
                manifest.getMainAttributes().putValue( "Main-Class", mainClassName );
            }
            /////////
            // DEBUG: print out all the Manifest's main attributes
            Attributes att = manifest.getMainAttributes();
            java.util.Set entries = att.entrySet();
            for ( java.util.Iterator i = entries.iterator(); i.hasNext(); )
            {
                System.out.println( "Manifest attribute:>> " + i.next().toString() );
            }


            //2. Create the file with output stream (and manifest)
            File jarFile = new File( jarDir, jarName );
            OutputStream fileOut = new FileOutputStream( jarFile );
            if ( manifest == null )
            {
                jarOut = new JarOutputStream( fileOut );
            }
            else
            {
                jarOut = new JarOutputStream( fileOut, manifest );
            }



            //	    todo for(File file: files){
            //		writeToJar(file, jarOut);
            //	    }
            jarOut.flush();
            jarOut.close();
            return true;
        }
        catch ( IOException ioe )
        {
            ioe.printStackTrace();
            return false;
        }
    }

    private static byte[] getFileBytes( File file ) throws IOException,
                                                           FileNotFoundException
    {
        long fileSize = file.length();
        byte[] arr = new byte[( int ) fileSize];

        FileInputStream fileIn = new FileInputStream( file );
        fileIn.read( arr );

        return arr;
    }

    private static void writeToJar( File file, JarOutputStream jarOut ) throws
                                                                        IOException,
                                                                        FileNotFoundException
    {
        byte[] fileBytes = getFileBytes( file );

        //TO DO: get a relative name for the file... (???)
        String fileName = file.getName();//RELATIVE NAME>>>>>
        System.out.println( "Adding file..." + fileName );
        //fileName = fileName.replaceAll("\\", "/"); (???)
        JarEntry entry = new JarEntry( fileName );
        jarOut.putNextEntry( entry );
        jarOut.write( fileBytes );
        jarOut.flush();

        if ( file.isDirectory() )
        {
            //		for(File child: file.listFiles()){
            //		writeToJar(child, jarOut);
            //	    }
        }

    }

    public static void main( String args[] )
    {
        if ( args.length < 3 )
        {
            System.out.println( "Usage: % java Jarrer <JAR name> "
                                + "<JAR source dir> <Manifest file> <file> [<file>...]" );
        }
        else
        {
            String jarName = args[0];
            File jarDir = new File( args[1] );
            File manifestFile = new File( args[2] );

            java.util.List list = new java.util.ArrayList();
            for ( int i = 3; i < args.length; i++ )
            {
                list.add( new File( args[i] ) );
            }
            File[] files = {};
            files = ( File[] ) list.toArray( files );

            boolean opSuccessful = jar( jarName, jarDir, manifestFile, files );
            System.out.println( "JAR operation for file "
                                + "\"" + jarDir.getName() + "\""
                                + " was "
                                + ( opSuccessful ? "" : "not " )
                                + "successful" );
        }
    }
}
