package idea.jartool.actions;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataKeys;
import com.intellij.openapi.roots.ModuleRootManager;
import com.intellij.openapi.roots.OrderRootType;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.wm.WindowManager;
import com.intellij.openapi.module.ModuleManager;
import com.intellij.openapi.module.Module;
import com.intellij.openapi.compiler.CompileStatusNotification;
import com.intellij.openapi.compiler.CompileContext;
import com.intellij.openapi.compiler.CompilerManager;
import com.intellij.openapi.util.Computable;
import com.intellij.openapi.util.JDOMUtil;
import com.intellij.find.FindProgressIndicator;
import com.intellij.usages.impl.UsageViewImplUtil;
import com.intellij.psi.PsiManager;
import com.intellij.psi.PsiClass;
import com.intellij.psi.search.GlobalSearchScope;
import idea.jartool.JarProject;
import idea.jartool.JarPlugin;
import idea.jartool.Library;
import org.apache.tools.ant.taskdefs.Manifest;
import org.jdom.Document;
import org.jdom.Element;

import javax.swing.*;
import java.io.File;
import java.util.Vector;

/**
 * Created by IntelliJ IDEA.
 * User: ruwan
 * Date: 20-Sep-2004
 * Time: 22:42:25
 */
public class AntAction extends AnAction
{
    String errorMsg = "";
    String projectDir = "";
    ModuleRootManager moduleRootManager;

    public AntAction()
    {
    }

    public void actionPerformed(AnActionEvent event)
    {
        //VirtualFile[] vFiles = (VirtualFile[]) event.getDataContext().getData(DataConstants.VIRTUAL_FILE_ARRAY);
        VirtualFile[] vFiles = event.getData(DataKeys.VIRTUAL_FILE_ARRAY);


        //Project project = (Project) event.getDataContext().getData(DataConstants.PROJECT);
        Project project = event.getData(DataKeys.PROJECT);

        if (vFiles != null)
        {
            for (VirtualFile vFile : vFiles)
            {
                if (vFile != null && vFile.getExtension() != null && vFile.getExtension().equals("jarprj"))
                {
                    JarProject jarProject = ApplicationManager.getApplication().getComponent(JarPlugin.class).getJarProject(vFile.getPath());

                    if (jarProject == null)
                    {
                        //this can happen if the .wasp file is not opened in the editor
                        jarProject = ApplicationManager.getApplication().getComponent(JarPlugin.class).loadJarProject(project, vFile);
                    }
                    else
                    {
                        //is opened in editor, save changes
                        ApplicationManager.getApplication().getComponent(JarPlugin.class).save(jarProject);
                    }

                    if (jarProject != null)
                    {
                        createAntBuildFile(jarProject, project);
                    }
                }
            }
        }
    }

    public void createAntBuildFile(final JarProject jarProject, final Project project)
    {
        try
        {
            //handling the output messages
            //this wasy is not needed when found the way to write in to the idea message window
            ApplicationManager.getApplication().getComponent(JarPlugin.class).showMessageBox(jarProject, true);
            ApplicationManager.getApplication().getComponent(JarPlugin.class).clearMessages(jarProject);
            ApplicationManager.getApplication().getComponent(JarPlugin.class).appendMessage(jarProject, "Creating Jar Archive..........");

            final FindProgressIndicator progressIndicator = new FindProgressIndicator(project, "");
            progressIndicator.setTitle("Creating Archive " + jarProject.getArchiveName() + "......");

                        
            errorMsg = "";
            final Runnable th = new Runnable()
            {
                public void run()
                {
                    progressIndicator.setIndeterminate(true);

                    try
                    {
                        showMessage("Creating Manifest..........");

                        Element manifest = new Element( "manifest");

                        if (!jarProject.getMainClass().trim().equals(""))
                        {
                            Element attribute = new Element("attribute");
                            attribute.setAttribute( "name", "Main-Class");
                            attribute.setAttribute( "value", jarProject.getMainClass());
                            manifest.addContent(attribute);
                        }

                        if (!jarProject.getVersion().trim().equals(""))
                        {
                            Element attribute = new Element("attribute");
                            attribute.setAttribute( "name", Manifest.ATTRIBUTE_MANIFEST_VERSION );
                            attribute.setAttribute( "value", jarProject.getVersion());
                            manifest.addContent(attribute);
                        }

                        String signedLibClasspath = "";

                        for (int i = jarProject.getLibraries().size() - 1; i >= 0; i--)
                        {
                            Library library = jarProject.getLibraries().elementAt(i);

                            if (library.isSigned())
                            {
                                signedLibClasspath += jarProject.getSignedLibraryPath() + File.separator + library.getFileName() + " ;  ";
                            }
                        }

                        if (signedLibClasspath.length() > 0)
                        {
                            Element attribute = new Element("attribute");
                            attribute.setAttribute( "name", Manifest.ATTRIBUTE_CLASSPATH );
                            attribute.setAttribute( "value", signedLibClasspath);
                            manifest.addContent(attribute);
                        }

                        /**
                         * if there is a path defined to eh sogned libraries, then the existing files of the
                         * library path should be removed before creating the new libs
                         */
                        if (jarProject.getSignedLibraryPath() != null && jarProject.getSignedLibraryPath().length() > 0)
                        {
                            File signedLibDir = new File(jarProject.getArchiveDir() + File.separator + jarProject.getSignedLibraryPath());

                            if (signedLibDir.exists())
                            {
                                File[] signedLibs = signedLibDir.listFiles();

                                for (File file : signedLibs)
                                {
                                    file.delete();
                                }
                            }
                            else
                            {
                                signedLibDir.mkdir();
                            }
                        }

                        Document antBuild = new Document();
                        Element antProject = new Element("project");
                        antProject.setAttribute( "name", jarProject.getArchiveName() );
                        antBuild.setRootElement(antProject);

                        Element delTarget = new Element("target");
                        delTarget.setAttribute( "name", "clear");
                        delTarget.setAttribute( "description", "Clear the Jar build");

                        Element deleteFile = new Element("delete");
                        deleteFile.setAttribute("file", jarProject.getArchiveDir() + File.separator + jarProject.getArchiveName());
                        delTarget.addContent(deleteFile);

                        antProject.addContent(delTarget);

                        Element target = new Element("target");
                        target.setAttribute( "name", "createJar");
                        target.setAttribute( "description", "Create the Jar build");
                        target.setAttribute( "depends", "clear");
                        antProject.addContent(target);

                        Element jarTask = new Element("jar");
                        jarTask.setAttribute("compress","true");
                        jarTask.setAttribute("destfile",jarProject.getArchiveDir() + File.separator + jarProject.getArchiveName());

                        target.addContent(jarTask);

                        jarTask.addContent(manifest);

                        showMessage("Adding Libraries.........");

                        /**
                         * Adding the libraries
                         *
                         * this need to be done in the reverse order in order to keep the ordering of libraries
                         * when creating the build, if the same class is available in more than one lib then the
                         * classes on the topof the list to be remain and not the last,
                         * when adding the jars from bottom to top the 1st added items will be replaced by the new entries
                         * hence the jar ordering will come to action
                         *
                         * also this need to be done prior to the project class selection as the project class may have
                         * class similar to the libraries, but project class shold be getting the priority
                         */
                        for (int i = jarProject.getLibraries().size() - 1; i >= 0; i--)
                        {
                            Library library = jarProject.getLibraries().elementAt(i);

                            showMessage("Adding : " + library.getPath());

                            if (library.isSigned())
                            {
//                                File targetFile = new File(jarProject.getArchiveDir() + File.separator + jarProject.getSignedLibraryPath() + File.separator + library.getFileName());
//
//                                File libraryFile = new File(library.getPath());
//
//
//                                FileOutputStream fout = new FileOutputStream(targetFile);
//                                FileInputStream fin = new FileInputStream(libraryFile);
//                                try
//                                {
//                                    // Allocate a buffer for reading the entry data.
//
//                                    byte[] buffer = new byte[1024];
//                                    int bytesRead;
//
//                                    // Read the entry data and write it to the output file.
//
//                                    while ((bytesRead = fin.read(buffer)) != -1)
//                                    {
//                                        fout.write(buffer, 0, bytesRead);
//                                    }
//
//                                    //System.out.println( ent.getName() + " extracted." );
//                                }
//                                catch (Exception e)
//                                {
//                                    e.printStackTrace();
//                                }
//                                finally
//                                {
//                                    fout.flush();
//                                    fout.close();
//                                    fin.close();
//                                }
                            }
                            else
                            {
                                Element zipFileSet = new Element("zipfileset");
                                zipFileSet.setAttribute("src", library.getPath());
                                jarTask.addContent(zipFileSet);
                            }
                        }

                        showMessage("Libraries Added.");
                        //loading class files from all the modules
                        showMessage("Loading Classes..........");

                        VirtualFile vfiles[] = moduleRootManager.getFiles(OrderRootType.CLASSES_AND_OUTPUT);

                        Vector<String> classEntries = new Vector<String>();

                        for (VirtualFile vfile : vfiles)
                        {
                            String vfilePath = vfile.getPresentableUrl();

                            //when search for Classes and Outouts from the Idea API it gives the jars as a output aswell
                            //they shold be exckuded from this
                            if (!vfilePath.trim().endsWith(".jar"))
                            {
                                if (jarProject.isIncludeAllProjectResources())
                                {
                                    Element fileSet = new Element("fileset");
                                    fileSet.setAttribute( "dir",vfilePath);
                                    jarTask.addContent(fileSet);
                                    //all the project files are selected, hence np need for check back from file
                                    showMessage("All Classes in " + vfilePath + " included.");
                                }
                                else
                                {
                                    //not all the classes are selected to the build, only selected to be added
                                    for (int i = 0; i < jarProject.getClasses().size(); i++)
                                    {
                                        String item = jarProject.getClasses().elementAt(i);

                                        if (item.startsWith(vfilePath))
                                        {
                                            //a selected class, add this
                                            progressIndicator.setText(vfilePath);
                                            String classEntry = item.substring(vfilePath.length() + 1);

                                            Element fileSet = new Element("fileset");
                                            fileSet.setAttribute( "src",classEntry);
                                            jarTask.addContent(fileSet);
//                                            fileSet.setIncludes(classEntry);
                                            //add to the class entries in order to find the sources of these if the source jar is to be built
                                            classEntries.addElement(classEntry);
//                                            found = true;
                                            showMessage("Class " + classEntry + " included.");
                                        }
                                    }
                                }
                            }
                        }

                        showMessage("Adding other resources.........");

                        //ading other resources,
                        for (int i = 0; i < jarProject.getResources().size(); i++)
                        {
                            showMessage("nAdding : " + jarProject.getResources().elementAt(i) + " ........");

                            progressIndicator.setText(jarProject.getResources().elementAt(i));
                            Element fileSet = new Element("zipfileset");

                            File file = new File(jarProject.getResources().elementAt(i));
                            fileSet.setAttribute("file",jarProject.getResources().elementAt(i));


                            if (file.getParent().startsWith(projectDir))
                            {
                                fileSet.setAttribute( "prefix", file.getParent().substring(projectDir.length() + (file.getParent().length() == projectDir.length() ? 0 : 1)));
                            }
                            jarTask.addContent(fileSet);
                        }

                        //buildign the source archive
                        if (jarProject.isBuildSourceArchive())
                        {
                            showMessage("Creating Source...................");

                            Element delSrcTarget = new Element("target");
                            delSrcTarget.setAttribute( "name", "clearSrc");
                            delSrcTarget.setAttribute( "description", "Clear the Source Jar");

                            Element delSrcFile = new Element("delete");
                            delSrcFile.setAttribute("file", jarProject.getSourceDir() + File.separator + jarProject.getSourceName());
                            delSrcTarget.addContent(delSrcFile);

                            antProject.addContent(delSrcTarget);

                            Element srcTarget = new Element("target");
                            srcTarget.setAttribute( "name", "createSrcJar");
                            srcTarget.setAttribute( "description", "Create the Source Jar");
                            srcTarget.setAttribute( "depends", "clearSrc");
                            antProject.addContent(srcTarget);

                            Element srcJarTask = new Element("jar");
                            srcJarTask.setAttribute("compress","true");
                            srcJarTask.setAttribute("destfile",jarProject.getSourceDir() + File.separator + jarProject.getSourceName());

                            srcTarget.addContent(srcJarTask);

                            vfiles = moduleRootManager.getFiles(OrderRootType.SOURCES);

                            for (VirtualFile vfile : vfiles)
                            {
                                //loaded source files, is directory return true for zips and jars
                                if (vfile.isDirectory() &&
                                    !(vfile.getPresentableUrl().endsWith(".zip") ||
                                        vfile.getPresentableUrl().endsWith(".jar")))
                                {
                                    if (jarProject.isIncludeAllProjectResources())
                                    {
                                        Element fileSet = new Element("fileset");
                                        fileSet.setAttribute( "dir",vfile.getPresentableUrl());
                                        srcJarTask.addContent(fileSet);

                                        //all classes included, hence include all the sources aswell
//                                        found = true;
                                        showMessage("All Source files in " + vfile.getPresentableUrl() + " included.");
                                    }
                                    else
                                    {
                                        //only sources of selected classes to be loaded, not all
                                        //this loads sources of the class file
                                        Vector<String> sourceToInclude = loadSrcOfClasses(project, classEntries);
                                        for (int i = 0; i < sourceToInclude.size(); i++)
                                        {
                                            String item = sourceToInclude.elementAt(i);

                                            if (item.startsWith(vfile.getPresentableUrl()))
                                            {
                                                progressIndicator.setText(vfile.getPresentableUrl());

                                                Element fileSet = new Element("fileset");
                                                fileSet.setAttribute( "src",item.substring(vfile.getPresentableUrl().length() + 1));
                                                srcJarTask.addContent(fileSet);

//                                                fileSet.setIncludes(item.substring(vfile.getPresentableUrl().length() + 1));
//                                                found = true;
                                                showMessage("Source file " + vfile.getPresentableUrl() + " included.");
                                            }
                                        }
                                    }
                                }
                            }

                            /**
                             * including the source jar files of the other libraries, if there are libraries with source files
                             * being selected to the project, these can be setup in the libraries setup and if the library is attached
                             * with a source then thy are automatically brought in here
                             */

                            showMessage("Adding external sources........");
                            for (int i = 0; i < jarProject.getSources().size(); i++)
                            {
                                showMessage("Source library " + jarProject.getSources().elementAt(i) + " included.");
                                progressIndicator.setText(jarProject.getSources().elementAt(i));
                                if (jarProject.isSingleSourceArchive())
                                {
                                    //sources of the libraries will be extracted and added to the source build as a single file
                                    Element zipFileSet = new Element("zipfileset");
                                    zipFileSet.setAttribute("src",jarProject.getSources().elementAt(i) );
                                    srcJarTask.addContent(zipFileSet);
                                }
                                else
                                {
                                    //source jar of the library fill be added as one entry to the source build
                                    Element fileSet = new Element("zipfileset");
                                    fileSet.setAttribute("file",jarProject.getSources().elementAt(i) );
                                    srcJarTask.addContent(fileSet);
                                }
                            }

                            //building the source file
                        }

                        JDOMUtil.writeDocument(antBuild, jarProject.getArchiveDir() + File.separator+jarProject.getArchiveName().substring(0, jarProject.getArchiveName().length() - 3 )+"xml" , "\n");
                    }
                    catch (Exception e)
                    {
                        e.printStackTrace();
                        errorMsg += "\n" + e.getMessage();
                    }

                    if (!errorMsg.equals(""))
                    {
                        ApplicationManager.getApplication().getComponent(JarPlugin.class).appendMessage(jarProject, "\nERROR....\n" + errorMsg);
                        JOptionPane.showMessageDialog(WindowManager.getInstance().suggestParentWindow(project), errorMsg, "", JOptionPane.ERROR_MESSAGE);
                    }

                    showMessage("Completed.................");
                }


                private void showMessage(String msg)
                {
                    ApplicationManager.getApplication().getComponent(JarPlugin.class).appendMessage(jarProject, "\n" + msg);
                    progressIndicator.setText2(msg);
                    WindowManager.getInstance().getStatusBar(project).setInfo(msg);
                }
            };


            ApplicationManager.getApplication().runWriteAction(new Runnable()
            {
                public void run()
                {
                    /**
                     * doing these things here since they should be callin ginside the runWriteAction()
                     */
                    projectDir = project.getBaseDir().getPath().replace('/', '\\');
                    ModuleManager moduleManager = ModuleManager.getInstance(project);
                    moduleRootManager = ModuleRootManager.getInstance(moduleManager.findModuleByName(project.getName()));

                    if (jarProject.isBuildBeforeArchiving())
                    {
                        //compiling before building tghe archive
                        CompileStatusNotification compileStatus = new CompileStatusNotification()
                        {

                            //IDEA 5.0 API
                            public void finished(boolean aborted, int errors, int warnings)
                            {
                                if (!aborted && errors == 0)
                                {

                                    //ApplicationManager.getApplication().runProcessWithProgressSynchronously( th, "Jar", true, project );
                                    //to enable background,
                                    UsageViewImplUtil.runProcessWithProgress(progressIndicator, th, new Thread());
                                    //ApplicationManager.getApplication().runWriteAction( th );
                                    ApplicationManager.getApplication().getComponent(JarPlugin.class).appendMessage(jarProject, "    Done.");
                                }
                            }

                            //IDEA 6.0 API
                            public void finished(boolean aborted, int errors, int warnings, CompileContext compileContext)
                            {
                                if (!aborted && errors == 0)
                                {

                                    //ApplicationManager.getApplication().runProcessWithProgressSynchronously( th, "Jar", true, project );
                                    //to enable background,
                                    UsageViewImplUtil.runProcessWithProgress(progressIndicator, th, new Thread());
                                    //ApplicationManager.getApplication().runWriteAction( th );
                                    ApplicationManager.getApplication().getComponent(JarPlugin.class).appendMessage(jarProject, "    Done.");
                                }
                            }
                        };

                        ApplicationManager.getApplication().getComponent(JarPlugin.class).appendMessage(jarProject, "\nCompiling..............");
                        Module module = ModuleManager.getInstance(project).findModuleByName(project.getName());
                        CompilerManager.getInstance(project).compile(module, compileStatus, true);
                    }
                    else
                    {
                        /**
                         * no compile, just build the archive
                         */
                        //ApplicationManager.getApplication().runProcessWithProgressSynchronously( th, "Jar", true, project );
                        UsageViewImplUtil.runProcessWithProgress(progressIndicator, th, new Thread());
                        //ApplicationManager.getApplication().runWriteAction( th );
                    }
                }
            });

        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    public Vector<String> loadSrcOfClasses(final Project project, final Vector<String> classEntries)
    {
        Vector<String> vec = new Vector<String>();

        final PsiManager psiManager = PsiManager.getInstance(project);

        final GlobalSearchScope gss = ApplicationManager.getApplication().runReadAction(new Computable<GlobalSearchScope>()
        {
            public GlobalSearchScope compute()
            {
                return GlobalSearchScope.moduleWithDependenciesScope(ModuleManager.getInstance(project).findModuleByName(project.getName()));
            }
        });

        for (int i = 0; i < classEntries.size(); i++)
        {
            String classFileEntry = classEntries.elementAt(i);
            final String classEntry = classFileEntry.substring(0, classFileEntry.length() - ".class".length()).replace('\\', '.');

            PsiClass psiClass = ApplicationManager.getApplication().runReadAction(new Computable<PsiClass>()
            {
                public PsiClass compute()
                {
                    return psiManager.findClass(classEntry, gss);
                }
            });


            if (psiClass != null)
            {
                if (!vec.contains(psiClass.getContainingFile().getVirtualFile().getPresentableUrl()))
                {
                    vec.addElement(psiClass.getContainingFile().getVirtualFile().getPresentableUrl());
                }
            }
        }

        return vec;
    }
}
