package idea.jartool.actions;

import com.intellij.openapi.actionSystem.*;
import com.intellij.openapi.fileEditor.FileEditorManager;
import com.intellij.openapi.module.Module;
import com.intellij.openapi.module.ModuleManager;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.roots.ModuleRootManager;
import com.intellij.openapi.util.JDOMUtil;
import com.intellij.openapi.vfs.LocalFileSystem;
import com.intellij.openapi.vfs.VirtualFile;
import com.intellij.openapi.wm.WindowManager;
import com.intellij.openapi.application.Application;
import com.intellij.openapi.application.ApplicationManager;
import idea.jartool.JarPlugin;
import idea.jartool.JarProject;
import org.jdom.Document;

import javax.swing.*;
import java.io.File;
import java.io.IOException;

/**
 * Created by IntelliJ IDEA.
 * User: ruwan
 * Date: 05-Jan-2005
 * Time: 15:07:46
 */
public class CreateJarAction extends AnAction
{
    public CreateJarAction()
    {
        super();
    }

    public void actionPerformed(AnActionEvent event)
    {
//        DataContext dataContext = event.getDataContext();
//        final Project project = (Project) dataContext.getData(DataConstants.PROJECT);

        
        final Project project =event.getData(DataKeys.PROJECT);

        if (project != null)
        {
            ApplicationManager.getApplication().runWriteAction(new Runnable()
            {
                public void run()
                {
                    String projectDir = project.getBaseDir().getPath();
                    String jarname = JOptionPane.showInputDialog(WindowManager.getInstance().suggestParentWindow(project), "Archive Name");

                    if (jarname != null)
                    {
                        final JarProject jarProject = new JarProject();

                        if (jarname.endsWith(".jar"))
                        {
                            jarname = jarname.substring(0, jarname.length() - ".jar".length());
                        }
                        else if (jarname.endsWith(".jarprj"))
                        {
                            jarname = jarname.substring(0, jarname.length() - ".jarprj".length());
                        }

                        jarProject.setArchiveName(jarname + ".jar");
                        jarProject.setSourceName(jarname + "-src.jar");
                        jarProject.setArchiveDir(projectDir);
                        jarProject.setSourceDir(projectDir);
                        jarProject.setProjectPathDirs(projectDir);

                        File jarPrjDir = new File(projectDir + File.separator + JarPlugin.ARCHIVE_DIR);
                        if (!jarPrjDir.exists())
                        {
                            jarPrjDir.mkdirs();
                        }

                        File jarPrjFile = new File(jarPrjDir.getPath() + File.separator + jarname + ".jarprj");

                        if (!jarPrjFile.exists())
                        {
                            try
                            {
                                jarPrjFile.createNewFile();
                            }
                            catch (IOException e)
                            {
                                e.printStackTrace();
                            }
                        }

                        try
                        {
                            Document doc = jarProject.createDoc();
                            JDOMUtil.writeDocument(doc, jarPrjFile, "\n");
                            VirtualFile v = LocalFileSystem.getInstance().refreshAndFindFileByIoFile(jarPrjFile);
                            FileEditorManager.getInstance(project).openFile(v, true);
                        }
                        catch (Exception e)
                        {
                            e.printStackTrace();
                            JOptionPane.showMessageDialog(WindowManager.getInstance().suggestParentWindow(project), e.getMessage(), "", JOptionPane.ERROR_MESSAGE);
                        }
                    }
                }
            });
        }
        else
        {
            //this canot be happen
            JOptionPane.showMessageDialog(WindowManager.getInstance().suggestParentWindow(project), "Could not find the Idea Project", "Error", JOptionPane.ERROR_MESSAGE);
        }

    }

    /**
     * @param virtualFile
     * @param project
     * @return the module of the file
     */
    public Module getModule(VirtualFile virtualFile, Project project)
    {
        Module[] modules = ModuleManager.getInstance(project).getModules();

        for (Module module : modules)
        {
            if (ModuleRootManager.getInstance(module).getFileIndex().isInSourceContent(virtualFile))
            {
                return module;
            }
        }

        return null;
    }
}
