/*
 * Created by IntelliJ IDEA.
 * User: Nethru
 * Date: Sep 3, 2002
 * Time: 4:46:07 PM
 * To change template for new interface use
 * Code Style | Class Templates options (Tools | IDE Options).
 */
package idea.tableutils;

import javax.swing.table.AbstractTableModel;
import java.util.Collections;
import java.util.Comparator;
import java.util.Vector;

public abstract class CGAbstractTableModel extends AbstractTableModel implements SortableTableModel
{
    protected Vector data;
    protected ColumnData[] columnData;

    public CGAbstractTableModel()
    {
        this.data = new Vector();
    }

    public CGAbstractTableModel( Vector data )
    {
        this.data = data;
    }

    public int getRowCount()
    {
        return data == null ? 0 : data.size();
    }

    public int getColumnCount()
    {
        return columnData.length;
    }

    public String getColumnName( int column )
    {
        return columnData[column].getTitle();
    }

    public ColumnData getColumn( int column )
    {
        return columnData[column];
    }

    public void removeAll()
    {
        if( data != null )
        {
            data.removeAllElements();
            fireTableDataChanged();
        }
    }

    public void setDataVector( Vector data )
    {
        this.data = data;
        fireTableDataChanged();
    }

    public Vector getDataVector()
    {
        return this.data;
    }

    public void insert( Object obj )
    {
        int row = data.size();
        data.insertElementAt( obj, row );
        fireTableDataChanged();
    }

    public boolean delete( int row )
    {
        if( row < 0 || row >= data.size() )
        {
            return false;
        }
        data.remove( row );
        fireTableDataChanged();
        return true;
    }

    public Object getRowObject( int nRow )
    {
        if( nRow < 0 || nRow >= data.size() )
        {
            return null;
        }
        return data.elementAt( nRow );
    }

    public boolean isCellEditable( int nRow, int nCol )
    {
        return true;
    }

    public boolean isValidated( boolean showMessages )
    {
        return true;
    }

    public boolean isInsertable()
    {
        return true;
    }

    public abstract Object getValueAtRow( Object rowObj, int nCol );

    public Object getValueAt( int rowIndex, int columnIndex )
    {
        if( data != null && data.size() > rowIndex )
        {
            Object obj = data.elementAt( rowIndex );
            return getValueAtRow( obj, columnIndex );
        }

        return "";
    }

    public void sortByColumn( int column, boolean isAscent )
    {
        try
        {
            Comparator comparator = null;
            final int direction = isAscent ? 1 : -1;
            final int col = column;

            comparator = new Comparator()
            {
                public int compare( Object o1, Object o2 )
                {
                    if( o1 == null && o2 == null )
                    {
                        return 0;
                    }
                    else if( o1 == null )
                    {
                        return -1;
                    }
                    else if( o2 == null )
                    {
                        return 1;
                    }
                    else
                    {
                        if( getValueAtRow( o1, col ) instanceof Boolean )
                        {
                            return direction * ((Comparable) getValueAtRow( o1, col ).toString() ).compareTo( ((Comparable)getValueAtRow( o2, col ).toString()) );
                        }
                        return direction * ( (Comparable) getValueAtRow( o1, col ) ).compareTo( ( (Comparable) getValueAtRow( o2, col ) ) );
                    }
                }
            };
            Collections.sort( data, comparator );
            fireTableDataChanged();
        }
        catch( Exception e )
        {
            System.out.println( "Column " + column );
            e.printStackTrace();
        }
    }

    public Vector getData()
    {
        return data;
    }

    public void setData( Vector data )
    {
        this.data = data;
        fireTableDataChanged();
    }
}
