package idea.tableutils;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableCellRenderer;
import java.awt.*;

public class CheckCellRenderer extends JCheckBox implements TableCellRenderer
{
    protected static Border m_noFocusBorder;

    public CheckCellRenderer()
    {
        super();
        m_noFocusBorder = new EmptyBorder(1, 2, 1, 2);
        setOpaque(true);
        setBorder(m_noFocusBorder);
    }

    public Component getTableCellRendererComponent(JTable table,
                                                   Object value, boolean isSelected, boolean hasFocus,
                                                   int row, int column)
    {
        if (value instanceof Boolean)
        {
            Boolean b = (Boolean) value;
            setSelected(b.booleanValue());
        }

        if (isSelected)
        {
            setForeground(table.getSelectionForeground());
            setBackground(table.getSelectionBackground());
        }
        else
        {
            if( table.getModel().isCellEditable( row, column ) )
            {
                setBackground(table.getBackground());
            }
            else
            {
                setBackground( UIManager.getColor( "Panel.background" ) );
            }
            setForeground(table.getForeground());
        }


        setFont(table.getFont());

		if ( hasFocus )
		{
			setBorder( UIManager.getBorder( "Table.focusCellHighlightBorder" ) );
			if ( table.isCellEditable( row, column ) )
			{
				super.setForeground( UIManager.getColor( "Table.focusCellForeground" ) );
				super.setBackground( UIManager.getColor( "Table.focusCellBackground" ) );
			}
		}
		else
		{
			setBorder( m_noFocusBorder );
		}

        return this;
    }
}